// Written 2000 by Michael Pfeiffer
#include "BeGVApp.h"

#include <stdio.h>
#include <stdlib.h>

// #define MORE_DEBUG 
#define DEBUG 0

#include <be/support/Debug.h>
#include <be/app/Message.h>
#include <be/storage/Entry.h>
#include <be/storage/Path.h>

// #include "TraceWindow.h"

void MakeEmpty(BListView *list) {
	int32 count = list->CountItems();
	if (count == 0) return;
	
	BListItem **items = new BListItem *[count];
	const BListItem **listItems = list->Items();
	for (int32 i = count-1; i >= 0; i--) {
		items[i] = (BListItem*)listItems[i];
	}
	list->MakeEmpty();
	for (int32 i = count-1; i >= 0; i--) {
		delete items[i];
	}
}

BMessage *BeGVWindow::GSDeviceMsg(const char *device) {
	BMessage *msg = new BMessage(MENU_ANTI_ALIASING);
	msg->AddString("device", device);
	return msg;
}

BeGVWindow::BeGVWindow(BRect aRect) 
	: BWindow(aRect, APPLICATION, B_TITLED_WINDOW, 0) {
	// initialize 
	mSettings.Load(APPLICATION);
	mOpenFilePanel = NULL;
	mDocument = NULL;
	mCurrentPage = 0;
	mDpi = mSettings.GetDpi();
	mState = S_NONE;
	mType = T_NONE;

//	mStdoutTracer = new OutputTracer(1, "stdout", &mSettings);
//	mStderrTracer = new OutputTracer(2, "stderr", &mSettings);
	
	SetPulseRate(0);
	// add menu bar
	BRect rect = BRect(0,0,aRect.Width(), aRect.Height());
	mMenubar = new BMenuBar(rect, "menu_bar");
	BMenu *menu; 

	menu = new BMenu("File");
	menu->AddItem(new BMenuItem("Open", new BMessage(MENU_FILE_OPEN), 'O'));
	menu->AddSeparatorItem();
	menu->AddItem(new BMenuItem("About ...", new BMessage(B_ABOUT_REQUESTED)));
	menu->AddSeparatorItem();
	menu->AddItem(new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q')); 
	mMenubar->AddItem(menu);

	menu = new BMenu("Page");
	menu->AddItem(new BMenuItem("Next", new BMessage(MENU_PAGE_NEXT)));
	menu->AddItem(new BMenuItem("Previous", new BMessage(MENU_PAGE_PREVIOUS)));
	menu->AddItem(new BMenuItem("Zoom In", new BMessage(MENU_ZOOM_IN), '+'));
	menu->AddItem(new BMenuItem("Zoom Out", new BMessage(MENU_ZOOM_OUT), '-'));
	BMenu *sub = new BMenu("Anti-Aliasing");
		sub->SetRadioMode(true);
		sub->AddItem(new BMenuItem("Off", GSDeviceMsg("beos")));
		sub->AddItem(new BMenuItem("2x2", GSDeviceMsg("bealpha2")));
		sub->AddItem(new BMenuItem("4x4", GSDeviceMsg("bealpha4")));
		sub->AddItem(new BMenuItem("2x2 text only", GSDeviceMsg("bealpht2")));
		sub->AddItem(new BMenuItem("4x4 text only", GSDeviceMsg("bealpht4")));
	menu->AddItem(sub);
	mMenubar->AddItem(menu);

	menu = new BMenu("Paper Size");
	menu->SetRadioMode(true);
	for (int32 i = 0; papersizes[i].name; i++) {
		if (papersizes[i].width && papersizes[i].height) {
			BMessage *msg = new BMessage(MENU_PAPER_SIZE);
			msg->AddInt32("width", papersizes[i].width);
			msg->AddInt32("height", papersizes[i].height);
			msg->AddString("name", papersizes[i].name);
			menu->AddItem(new BMenuItem(papersizes[i].name, msg));
		}
	}
	mMenubar->AddItem(menu);


	menu = new BMenu("Zoom");
	menu->SetRadioMode(true);
	for (int32 i = -3; i < 10; i++) {
		char buffer[40];
		int32 dpi =  72 + 20*i;
		sprintf(buffer, "%d%%", (int)(100 * dpi / 72));
		BMessage *msg = new BMessage(MENU_ZOOM);
		msg->AddInt32("dpi", dpi);
		menu->AddItem(new BMenuItem(buffer, msg));
	}
	mMenubar->AddItem(menu);

	AddChild(mMenubar);
	rect.Set(0, mMenubar->Bounds().Height(), 50 - B_V_SCROLL_BAR_WIDTH, mMenubar->Bounds().Height() + 16);
	mTypeText = new BStringView(rect, "name", "");
	AddChild(mTypeText);
	
	rect.Set(2, mMenubar->Bounds().Height()+17, 50 - B_V_SCROLL_BAR_WIDTH, aRect.Height()-2);
	
	mPagesView = new BListView(rect, "pagesList", B_SINGLE_SELECTION_LIST, B_FOLLOW_LEFT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW | B_FRAME_EVENTS | B_FULL_UPDATE_ON_RESIZE);
	mPagesView->SetSelectionMessage(new BMessage(PAGE_SELECTED));
	AddChild(new BScrollView ("pageScrollView", mPagesView, B_FOLLOW_LEFT | B_FOLLOW_TOP_BOTTOM, 0, false, true));
	// add Ghostscript view
	rect.Set(54, mMenubar->Bounds().Height()+1, aRect.Width() - B_V_SCROLL_BAR_WIDTH, aRect.Height() - B_H_SCROLL_BAR_HEIGHT);
	mMainView = new BeGVView(rect, mSettings.GetDpi(), mSettings.GetPaperWidth(), mSettings.GetPaperHeight());
	AddChild(new BScrollView ("scrollView", mMainView, B_FOLLOW_ALL, 0, true, true));
	mMainView->SetDevice(mSettings.GetDevice());
	// make window visible
	mMainView->Resize();
	mMainView->MakeFocus();
	MoveTo(mSettings.GetWindowPosition());
	ResizeTo(mSettings.GetWindowWidth(), mSettings.GetWindowHeight());
	Show();

}

#ifdef MORE_DEBUG
static char *states[] = { "S_NONE", "S_RENDERING", "S_WAITING", "S_ABORTING", "S_PROCESSING" };
#endif

void BeGVWindow::SetState(int32 state) {
#ifdef MORE_DEBUG
	fprintf(stderr, "state old= %s new= %s\n", states[mState], states[state]);
#endif
	mState = state;
	switch (state) {
	case S_NONE: 
	case S_WAITING:
	case S_PROCESSING: 
		mMainView->StopRefreshingPulse();
		mMainView->Invalidate();
		break;
	case S_RENDERING:
		mMainView->StartRefreshingPulse();
		break;
	case S_ABORTING:
		break;
	}
}

static char *types[] = { "NONE", "EPS", "PS DSC", "PS plain" };

void BeGVWindow::SetType(int32 type) {
#ifdef MORE_DEBUG
	fprintf(stderr, "type old= %s new= %s\n", types[mType], types[type]);
#endif
	mType = type;
	mTypeText->SetText(types[type]);
}

void BeGVWindow::ContinuePage() {
	if (mState == S_WAITING) {
		if (mType == T_DSC) {
			//if ((mPendingCmd == CMD_SET_PAGE) && (mPendingValue == mCurrentPage + 1)) {
			//	mMainView->ContinuePage();
			//	SetState(S_RENDERING);
			//} else {
				mMainView->NextPage(true);
				SetState(S_ABORTING);
			//}
		} else if (mType == T_EPS) { // this must not happend
			mMainView->NextPage(true);
			SetState(S_ABORTING);
		} else if (mType == T_PS) {
			if (mPendingCmd == CMD_SET_PAGE) {
				mPendingCmd = CMD_NONE;
				mMainView->NextPage();
				SetState(S_RENDERING);
			} else {
				mMainView->NextPage(true);
				SetState(S_ABORTING);
			}
		}
	} else if (mState == S_PROCESSING) {
		mMainView->Stop();
		SetState(S_ABORTING);
	}
}

void BeGVWindow::ActivateCommand() {
	if (mPendingCmd != CMD_NONE) {
		if (mState == S_WAITING) { // a command is waiting, so abort GS
			ContinuePage();
		} else if (mState == S_NONE) {
			switch (mPendingCmd) {
			case CMD_SET_PAGE: SetPage(mPendingValue);
				break;
			case CMD_SET_DPI: SetDpi(mPendingValue);
				break;
			case CMD_OPEN_FILE_PANEL: OpenFilePanel();
				break;
			case CMD_OPEN_FILE: OpenFile(&mPendingRefs);
				break;
			case CMD_QUIT: PostMessage(B_QUIT_REQUESTED);
			}
		} else if (mState == S_PROCESSING) { // GS is waiting for a new command
			if ((mType == T_EPS) || (mType == T_PS)) {
				mMainView->Stop();
				SetState(S_ABORTING);
			} 
		}
	}
}

bool BeGVWindow::Command(int32 cmd, int32 value) {
	if (mState == S_NONE) {
		mPendingCmd = CMD_NONE;
		return true;
	} else {
		mPendingCmd = cmd; mPendingValue = value;
		ContinuePage();
		return false;
	}	
}

void BeGVWindow::MessageReceived(BMessage *message) {
	switch(message->what) {
	case B_SIMPLE_DATA:	OpenFile(message);
		break;
	case MENU_FILE_OPEN: OpenFilePanel();
		break;
	case B_REFS_RECEIVED: OpenFile(message);
		break;
	case Ghostscript::PAGE_NOTIFY: SetState(S_WAITING); ActivateCommand();
		break;
	case Ghostscript::PAGE_CONT_NOTIFY: SetState(S_RENDERING);
		break;
	case Ghostscript::SYNC_NOTIFY: mMainView->Invalidate();
		break;
	case Ghostscript::SIZE_NOTIFY: SizeChanged(message);
		break;
	case Ghostscript::CMD_ERROR_NOTIFY: {
		BAlert *error = new BAlert("Error", "There was an error rendering this page. The error message was written to stderr.", "Close", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
		error->Go();
		}
		break;
	case Ghostscript::CMD_NOTIFY: SetState(S_PROCESSING);
		break;
	case Ghostscript::EXIT_NOTIFY: SetState(S_NONE); ActivateCommand();
		break;
	case MENU_PAGE_NEXT: SetPage(mCurrentPage + 1);
		break;
	case MENU_PAGE_PREVIOUS: SetPage(mCurrentPage - 1);
		break;
	case MENU_ZOOM_IN: SetDpi((int32)(mDpi * 1.2));
		break;
	case MENU_ZOOM_OUT: SetDpi((int32)(mDpi / 1.2));
		break;
	case PAGE_SELECTED: PageSelected(message);
		break;
	case MENU_PAPER_SIZE: PaperSizeSelected(message);
		break;
	case MENU_ZOOM: ZoomSelected(message);
		break;
	case MENU_ANTI_ALIASING: AASelected(message);
		break;
	case B_ABOUT_REQUESTED:
		AboutRequested();
		break;
	default:
		BWindow::MessageReceived(message);
	}
}


bool BeGVWindow::QuitRequested() {
	if (Command(CMD_QUIT, 0)) {
		mSettings.Save(APPLICATION);
//	    delete mStdoutTracer; mStdoutTracer = NULL;
//	    delete mStderrTracer; mStderrTracer = NULL;
		be_app->PostMessage(B_QUIT_REQUESTED);
		return true;
	} else {
		return false;
	}
}

void BeGVWindow::FrameMoved(BPoint p) {
	mSettings.SetWindowPosition(p);
}

void BeGVWindow::FrameResized (float width, float height) {
	mMainView->Resize();
	mSettings.SetWindowSize(width, height);
}

void BeGVWindow::OpenFilePanel() {
	if (!Command(CMD_OPEN_FILE_PANEL, 0)) return;
	
	if (mOpenFilePanel == NULL) {
		BMessenger messenger(NULL, this);
		mOpenFilePanel = new BFilePanel (B_OPEN_PANEL,
						&messenger, NULL, B_FILE_NODE, true, NULL, NULL);
	}
	mOpenFilePanel->Show();
}

bool BeGVWindow::IsPDF(BString *file) {
	int32 p = file->FindLast('.');
	if (p != B_ERROR) {
		BString ext;
		file->CopyInto(ext, p + 1, file->Length() - p -1);
		ext.ToUpper();
		return ext == "PDF";
	}
	return false;
}

bool BeGVWindow::PDF2PS(BString *pdfFile) {
	BString psFile("/boot/home/pdf.ps");
	char buffer[2048];
	sprintf(buffer, "gs -dNODISPLAY -dQUIET -sPDFname=\"%s\" -sDSCname=\"%s\" pdf2dsc.ps -c quit",
		pdfFile->String(), psFile.String());
	int ret = system(buffer);
	fprintf(stderr, "%s\n %d\n", buffer, ret);
	mFileName = psFile;
	return ret >= 0;
}

void BeGVWindow::OpenFile(BMessage *refs) {
	type_code type;
	int32 count, i;
	entry_ref ref;

	if (mState == S_NONE) {
		mPendingCmd = CMD_NONE;
	} else {
		mPendingCmd = CMD_OPEN_FILE; mPendingValue = 0;
		mPendingRefs = *refs;
		ContinuePage();
		return;
	}	

	mMainView->MakeFocus();
	
	refs->GetInfo ("refs", &type, &count);
	if ( type != B_REF_TYPE ) {
		return;
	}
	
	Close();

	for ( i = --count ; i >= 0; i-- ) {
		if ( refs->FindRef("refs", i, &ref ) == B_OK ) {
			BEntry entry(&ref);
			BPath path;
			entry.GetPath(&path);
			mFileName = path.Path();
			if (IsPDF(&mFileName)) {
				if (!PDF2PS(&mFileName)) {
					BAlert *error = new BAlert("Error", "Could not convert the PDF-file to PostScript.", "Close", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
					error->Go();
					break;
				}
			}
			FILE *file = fopen(mFileName.String(), "r+b");
			if (file) {
				char *filename_dsc;
				char *cmd_scan_pdf = "";
				char *filename_unc;
				char *cmd_uncompress = NULL;

				mDocument = psscan(&file, (char*)mFileName.String(), (char*)mFileName.String(), 
					&filename_dsc, cmd_scan_pdf, 
					&filename_unc, cmd_uncompress, 0);
				fclose(file);

				if (mDocument) {
					if (GetNumPages() > 0) {
						mPageList.SetTo(' ', GetNumPages());
						SetType(T_DSC);
					} else if (mDocument->epsf) {
						SetType(T_EPS);
					} else {
						psfree(mDocument); mDocument = NULL;
						continue;
					}
				} else {
					//BAlert *error = new BAlert("Error", "Could not read the document structuring convention (DSC) comments from PS-file.", "Close", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
					//error->Go();
					// mMainView->SetPage(mFileName.String());
					SetType(T_PS);
				}
				FileOpened();
				return;
			}
		}
	}	
}

void BeGVWindow::Close() {
	if (mDocument) {
		MakeEmpty(mPagesView);
		psfree(mDocument);
		mDocument = NULL;
		mPageList = "";
	}
	mType = T_NONE;
}

void BeGVWindow::FileOpened() {
	if (mDocument) {
		Media m = mDocument->default_page_media;
		if (m) {
			mMainView->SetPaperSize(m->width, m->height);
		}
	}
	
	MakeEmpty(mPagesView);
	if (GetNumPages() > 0) {
		BList list;
		for (uint32 i = 0; i < mDocument->numpages; i++) {
			if (mDocument->pages[i].label) {
				list.AddItem(new BStringItem(mDocument->pages[i].label));
			} else {
				char buffer[20];
				sprintf(buffer, "P%d", (int)i);
				list.AddItem(new BStringItem(buffer));
			}
		}
		mPagesView->AddList(&list);
	}

	SetPage(0);
	mMainView->Resize();
}

void BeGVWindow::SetPage(int32 pageNo) {
	if (!Command(CMD_SET_PAGE, pageNo)) return;	
	
	if (mType == T_DSC) {
		if ((pageNo >= 0) && (pageNo < GetNumPages())) {
			if ((mCurrentPage >= 0) && (mCurrentPage < GetNumPages())) {
				mPageList[mCurrentPage] = ' ';
			}
			mCurrentPage = pageNo;
			mPagesView->Select(mCurrentPage);
			mPagesView->ScrollToSelection();
			mPageList[mCurrentPage] = '*';
			FILE *file = fopen("/boot/home/gvpage.ps", "w+b");
			if (file) {
				pscopydoc(file, (char*)mFileName.String(), mDocument, (char*)mPageList.String());
				fclose(file);
				mMainView->SetPage("/boot/home/gvpage.ps");
				SetState(S_RENDERING);
			}
		}
	} else if (mType == T_EPS) {
		if (mState == S_NONE) {
			mMainView->SetPage(mFileName.String());
			SetState(S_RENDERING);
		}
	} else if (mType == T_PS) { // maybe a PS-file without DSC comments
		if (mState == S_NONE) {
			mMainView->SetPage(mFileName.String());
			SetState(S_RENDERING);
		}
	}
}

void BeGVWindow::SetDpi(int32 dpi) {
	if (!Command(CMD_SET_DPI, dpi)) return;	

	if ((dpi >= 5) && (dpi <= 400)) {
		mSettings.SetDpi(dpi);
		mMainView->SetDpi(dpi);
		mDpi = dpi;
		SetPage(mCurrentPage);
		mMainView->Resize();
	}
}

void BeGVWindow::PageSelected(BMessage *msg) {
	int32 selected = mPagesView->CurrentSelection(0);
	if (selected >= 0) {
		SetPage(selected);
	}
}

void BeGVWindow::PaperSizeSelected(BMessage *msg) {
	BString name; int32 w, h;
	if ((B_OK == msg->FindString("name", &name)) &&
		(B_OK == msg->FindInt32("width", &w)) &&
		(B_OK == msg->FindInt32("height", &h))) {
		mMainView->SetPaperSize(w, h);
		mSettings.SetPaperWidth(w);
		mSettings.SetPaperHeight(h);
		mMainView->Resize();
	}
}

void BeGVWindow::ZoomSelected(BMessage *msg) {
	int32 dpi;
	if (B_OK == msg->FindInt32("dpi", &dpi)) {
		SetDpi(dpi);
	}
}

void BeGVWindow::AASelected(BMessage *msg) {
	BString device;
	if (B_OK == msg->FindString("device", &device)) {
		mSettings.SetDevice(device.String());
		mMainView->SetDevice(device.String());
		SetPage(mCurrentPage);
	}	
}

void BeGVWindow::SizeChanged(BMessage *msg) {
	int32 w, h;
	if ((B_OK == msg->FindInt32("width", &w)) &&
		(B_OK == msg->FindInt32("height", &h))) {
		//fprintf(stderr, "Size w = %d, h = %d\n", w, h);
		mMainView->SetBitmapSize(w, h);
	}	
}

void BeGVWindow::AboutRequested() {
	BAlert *about = new BAlert(APPLICATION, 
		APPLICATION " " VERSION "\nThis program is freeware.\n\n"
		"Written 2000,2002.\n\n"
		"By Michael Pfeiffer.\n\n"
		"EMail: michael.pfeiffer@utanet.at.","Close");
		about->Go();
}

SimpleGhostview::SimpleGhostview() : BApplication("application/x-mp.BeGV") {
	// set up a rectangle and instantiate a new window
	BRect aRect(0, 0, 800, 600);
	aRect.OffsetTo(20, 25);
	window = NULL;
	window = new BeGVWindow(aRect);		
}

void SimpleGhostview::RefsReceived(BMessage *msg) {
	if (window->Lock()) {
		window->OpenFile(msg);
		window->Unlock();
	}
}

int main(int argc, char *argv[]) { 
	be_app = NULL;
	if (!Ghostscript::LoadGSDll()) return 0;
	SimpleGhostview app;
	be_app->Run();
	return 0;
}