// Written 2000 by Michael Pfeiffer

#ifndef SIMPLE_GHOSTVIEW_H
#define SIMPLE_GHOSTVIEW_H

#include <be/app/Application.h>
#include <InterfaceKit.h>
#include <be/storage/FilePanel.h>
#include <be/support/String.h>

#include <psscanner.h>

#include "MsgConsts.h"
#include "BeGVView.h"
#include "GlobalSettings.h"
//#include "TraceWindow.h"

#define APPLICATION "BeGV"
#define VERSION "0.1.2"

class BeGVWindow : public BWindow {
	BMenuBar *mMenubar;
	BeGVView *mMainView;
	BListView *mPagesView;
	BFilePanel *mOpenFilePanel;
	
	BString mFileName;
	Document mDocument;
	BString mPageList; // * .. selected 
	int32 mCurrentPage;
	int32 mDpi;

	GlobalSettings mSettings;

//	OutputTracer  *mStdoutTracer;
//	OutputTracer  *mStderrTracer;

	int32 mPendingCmd, mPendingValue;
	BMessage mPendingRefs;
	enum {
		CMD_NONE,
		CMD_SET_PAGE,
		CMD_SET_ZOOM,
		CMD_SET_DPI,
		CMD_OPEN_FILE_PANEL,
		CMD_OPEN_FILE, // data in mPendingRefs
		CMD_QUIT
	};

	int32	mState; 
	enum {
		S_NONE,
		S_RENDERING,
		S_WAITING,
		S_ABORTING,
		S_PROCESSING
	};

	int32 mType;
	enum {
		T_NONE,
		T_EPS, // with DSC
		T_DSC, // with showpage
		T_PS // EPS or multiple pages
	};

	BStringView *mTypeText;

	void PageSelected(BMessage *msg);
	void PaperSizeSelected(BMessage *msg);
	void ZoomSelected(BMessage *msg);
	void AASelected(BMessage *msg);
	
	bool IsPDF(BString *file);
	bool PDF2PS(BString *file);

	BMessage *GSDeviceMsg(const char *device);
	void SizeChanged(BMessage *msg);
	
	void ContinuePage();
	bool Command(int32 cmd, int32 value);
	void ActivateCommand();
public:
	BeGVWindow(BRect);

	void SetState(int32 state);
	void SetType(int32 type);
		
	void OpenFilePanel();
	void OpenFile(BMessage *refs);
	void FileOpened(); // new file was opened
	
	void Close();
	int32 GetNumPages() { return mDocument ? mDocument->numpages : 0; };
	BString *GetPageList() { return &mPageList; };
	Document GetDocument() { return mDocument; };
	int32 GetCurrentPage() { return mCurrentPage; };
	void SetPage(int32 pageNo);
	void SetDpi(int32 dpi);
	
	// called from BeGVView
	void NextPage() { SetPage(mCurrentPage + 1); };
	void PreviousPage() { SetPage(mCurrentPage - 1); };
	void ZoomIn() { SetDpi((int32)(mDpi * 1.2)); };
	void ZoomOut() { SetDpi((int32)(mDpi / 1.2)); };
	
	bool QuitRequested();
	virtual void FrameMoved(BPoint p);
	virtual void FrameResized (float width, float height);
	void AboutRequested();	
	void MessageReceived(BMessage *message);
};

class SimpleGhostview : public BApplication {
public:
	BeGVWindow *window;
	SimpleGhostview();
	void RefsReceived(BMessage *msg);
};

#define my_app ((SimpleGhostview*)be_app)
#endif
