// Written 2000 by Michael Pfeiffer

#include "BeGVView.h"
#include "BeGVApp.h"
#include <stdio.h>
#include <be/interface/Window.h>

BeGVView::BeGVView(BRect rect, int32 dpi, int32 width, int32 height) 
  : BView(rect, NULL, 
	B_FOLLOW_ALL_SIDES, 
	B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE | B_PULSE_NEEDED) {

	SetViewColor(B_TRANSPARENT_COLOR);
	SetHighColor(255, 255, 255);
	bpp = 32; maxSize = 32; // 32 MB
	drawTime = 0;
	
	realPaperWidth = width; realPaperHeight = height;
	this->dpi = dpi;
	SetPaperSize(realPaperWidth, realPaperHeight);
	gs.SetDpi(72, 72);
	gs.SetPaperSize(paperWidth, paperHeight);
	mLeft = mTop = 0;
}

void BeGVView::AttachedToWindow() {
	gs.SetLooper((BLooper*)Window());
}

void BeGVView::DetachedFromWindow() {
	gs.NextPage(true);
}

void BeGVView::Pulse() {
	if ((system_time() - drawTime) > 1000000) {
		Invalidate();
	}
}

void BeGVView::StartRefreshingPulse() {
	Window()->SetPulseRate(200000);
}

void BeGVView::StopRefreshingPulse() {
	Window()->SetPulseRate(0);
}

void BeGVView::Draw(BRect updateRect) {
	BRect from(0, 0, min_c(paperWidth, bitmapWidth), min_c(paperHeight-1, bitmapHeight));
	from.right --; from.bottom --;
	BRect to(from); to.OffsetBy(mLeft, mTop);
//	if (!gs.Draw(this, updateRect.OffsetByCopy(-mLeft, -mTop), updateRect)) {
	if (!gs.Draw(this, from, to)) {
//		BRect r(0, 0, bitmapWidth, bitmapHeight);
//		r.OffsetBy(mLeft, mTop);
		FillRect(to);
	} else {
		if (bitmapWidth < paperWidth) {
			if (bitmapHeight < paperHeight) {
				BRect r(bitmapWidth, 0, paperWidth-1, paperHeight-1);
				r.OffsetBy(mLeft, mTop);
				FillRect(r);
				r.Set(0, bitmapHeight, bitmapWidth, paperHeight-1);
				r.OffsetBy(mLeft, mTop);
				FillRect(r);
			} else {
				BRect r(bitmapWidth, 0, paperWidth-1, paperHeight-1);
				r.OffsetBy(mLeft, mTop);
				FillRect(r);
			}
		} else if (bitmapHeight < paperHeight) {
			BRect r(0, bitmapHeight, paperWidth-1, paperHeight-1);
			r.OffsetBy(mLeft, mTop);
			FillRect(r);
		}
	}
	BRect rect(Bounds());
	float right = mLeft + paperWidth-1, bottom = mTop + paperHeight-1;
	SetLowColor(128, 128, 128, 0);
	if (rect.left < mLeft) {
		FillRect(BRect(rect.left, rect.top, mLeft - 2, rect.bottom), B_SOLID_LOW);
	}
	if (rect.top < mTop) {
		FillRect(BRect(rect.left, rect.top, rect.right, mTop - 2), B_SOLID_LOW);
	}
	if (right < rect.right) {
		FillRect(BRect(right + 2, rect.top, rect.right, rect.bottom), B_SOLID_LOW);
	}
	if (bottom < rect.bottom) {
		FillRect(BRect(rect.left, bottom + 2, rect.right, rect.bottom), B_SOLID_LOW);
	}

	SetLowColor(0, 0, 0, 0);
	StrokeRect(BRect(mLeft - 1, mTop - 1, right + 1, bottom + 1), B_SOLID_LOW);

	drawTime = system_time();
}

BPoint BeGVView::CorrectMousePos(const BPoint point) {
	BPoint p(point);
	p.x -= mLeft;
	p.y -= mTop;
	return p;
}

void BeGVView::MouseDown(BPoint point) {
	MakeFocus();
	uint32 buttons;
	GetMouse(&point, &buttons, false);
	CorrectMousePos(point);
/*
	if (buttons == B_PRIMARY_MOUSE_BUTTON) {
		if ((modifiers() & B_CONTROL_KEY)) {
			buttons = B_SECONDARY_MOUSE_BUTTON; // simulate secondary button 
		} else if ((modifiers() & B_SHIFT_KEY)) {
			buttons = B_TERTIARY_MOUSE_BUTTON; // simulate tertiary button 
		}
	}
*/
	switch (buttons) {
	case B_PRIMARY_MOUSE_BUTTON:  
			// drag view
			mMouseDown = B_PRIMARY_MOUSE_BUTTON;
			SetMouseEventMask(B_POINTER_EVENTS);
			mMousePosition = ConvertToScreen(point);
		break;
	}
}

void BeGVView::MouseMoved(BPoint point, uint32 transit, const BMessage *message) {
	if (mMouseDown == B_PRIMARY_MOUSE_BUTTON) { // move view
//		SkipMouseMoveMsgs();

		BPoint mousePosition;
		uint32 buttons;
		BPoint offset;
		float x, y, r_min, r_max;
		BScrollBar *scroll;

		GetMouse(&mousePosition, &buttons, false);
		point = ConvertToScreen(mousePosition);
		offset = point - mMousePosition;
		mMousePosition = point;

		scroll = ScrollBar(B_HORIZONTAL);
		scroll->GetRange(&r_min, &r_max);
		x = min_c(r_max, max_c(scroll->Value() - offset.x, r_min));

		scroll = ScrollBar(B_VERTICAL);
		scroll->GetRange(&r_min, &r_max);
		y = min_c(r_max, max_c(scroll->Value() - offset.y, r_min));

		ScrollTo(x, y);
		if ((buttons & B_PRIMARY_MOUSE_BUTTON) == 0) {
			MouseUp(mousePosition);
		}
	}
}

void BeGVView::MouseUp(BPoint point) {
	mMouseDown = 0;
}

void BeGVView::CenterPage() {
	BRect bounds(Bounds());
	if (bounds.Width() + 1 > paperWidth) { // center page horizontally
		mLeft = (int32)((bounds.Width() - paperWidth) / 2);
	} else {
		mLeft = 0;
	}

	if (bounds.Height() + 1 > paperHeight) { // center page vertically
		mTop = (int32)((bounds.Height() - paperHeight) / 2);
	} else {
		mTop = 0;
	}
}

void BeGVView::FixScrollbars()
{
	BRect frame = Bounds();
	BScrollBar * scroll;
	float x, y;
	float bigStep, smallStep;
	int w = paperWidth - 1, h = paperHeight - 1; 
	x = w - frame.Width();
	if (x < 0.0) {
		x = 0.0;
	}
	y = h - frame.Height();
	if (y < 0.0) {
		y = 0.0;
	}
	
	scroll = ScrollBar (B_HORIZONTAL);
	scroll->SetRange (0.0, x);
	scroll->SetProportion ((w - x) / w);
	bigStep = frame.Width() - 2;
	smallStep = bigStep / 10.;
	scroll->SetSteps (smallStep, bigStep);

	scroll = ScrollBar (B_VERTICAL);
	scroll->SetRange (0.0, y);
	scroll->SetProportion ((h - y) / h);
	bigStep = frame.Height() - 2;
	smallStep = bigStep / 10.;
	scroll->SetSteps (smallStep, bigStep);
}

void BeGVView::Resize() {
	CenterPage();
	FixScrollbars();
	Invalidate();
}

bool BeGVView::CheckMaxSize(int32 w, int32 h, int32 dpi, int32 bpp) {
	int64 size = (((int64)w) * h * dpi * dpi / (72 * 72)) * (bpp / 8);
	//fprintf(stderr, "w = %d, h = %d dpi = %d bpp = %d\n", w, h, dpi, bpp);
	size /= 1024 * 1024;
	//fprintf(stderr, "size = %d MBytes\n", size);
	return size < maxSize;
}

void BeGVView::CalcSizes() {
	paperWidth = dpi * realPaperWidth / 72;
	paperHeight = dpi * realPaperHeight / 72;
}

void BeGVView::SetPaperSize(int32 width, int32 height) { 
	if (CheckMaxSize(width, height, dpi, bpp)) {
		realPaperWidth = width; realPaperHeight = height;
		CalcSizes();
	}
};

void BeGVView::SetDpi(int32 dpi) {
	if (CheckMaxSize(paperWidth, paperHeight, dpi, bpp)) {
		this->dpi = dpi;
		CalcSizes();
	}
}

void BeGVView::SetBitmapSize(int32 width, int32 height) {
	bitmapWidth = width; bitmapHeight = height;
}

void BeGVView::SetPage(const char *filename) {
	gs.Stop();
	gs.SetDpi(dpi, dpi);
	gs.SetPaperSize(paperWidth, paperHeight);
//	gs.GetBitmapSize(bitmapWidth, bitmapHeight);
	gs.Start(); 
	char s[512];
	sprintf(s, "(%s) run\n", filename);
	gs.Send(s, strlen(s));
}

void BeGVView::NextPage(bool stop) {
	if (gs.WaitsForPage()) {
		gs.NextPage(stop);
		if (stop) {
			gs.Stop();
		}
	}
}

void BeGVView::KeyDown (const char * bytes, int32 numBytes)
{
	switch (*bytes) {
	case B_PAGE_UP: window()->PreviousPage();
		break;
	case B_PAGE_DOWN: window()->NextPage();
		break;
	case '+': window()->ZoomIn();
		break;
	case '-': window()->ZoomOut();
		break;
	case B_SPACE:
	case B_BACKSPACE:
		break;
	case B_DOWN_ARROW:
	case B_UP_ARROW:
		break;
	case B_LEFT_ARROW:
	case B_RIGHT_ARROW:
		break;
	case B_HOME:
		break;
	case B_END:
		break;
	default:
		BView::KeyDown (bytes, numBytes);
		break;
	}
}