// Written 2000 by Michael Pfeiffer

#ifndef VIEW_H
#define VIEW_H

#include <be/interface/View.h>
#include <be/interface/ScrollView.h>

#include <GhostscriptWrapper.h>

class BeGVWindow;

class BeGVView : public BView {
	Ghostscript gs;	
	int32 realPaperWidth, realPaperHeight, // 72 in
		dpi,
		paperWidth, paperHeight,  
		bitmapWidth, bitmapHeight;

	int8 bpp; // bits per pixel
	int32 maxSize; // in MB

	bigtime_t drawTime;
	
	int32 mMouseDown;
	int32 mLeft, mTop;
	BPoint mMousePosition;
	BPoint CorrectMousePos(const BPoint point);
	void CenterPage();
	void FixScrollbars();
	bool CheckMaxSize(int32 w, int32 h, int32 dpi, int32 bpp);
	void CalcSizes();
public:
	BeGVWindow *window() { return (BeGVWindow*)Window(); };

	BeGVView(BRect rect, int32 dpi, int32 paperWidth, int32 paperHeight);
	void Draw(BRect updateRect);
	void MouseDown(BPoint point);
	void MouseMoved(BPoint point, uint32 transit, const BMessage *message);
	void MouseUp(BPoint point);
	void KeyDown (const char * bytes, int32 numBytes);
	
	void AttachedToWindow();
	void DetachedFromWindow();
	void Pulse();

	void Resize();
	
	void StartRefreshingPulse();
	void StopRefreshingPulse();
	
	void SetDevice(const char *device) { gs.SetDevice(device); };
	void SetPage(const char *filename);
	void NextPage(bool stop = false);
	void SetDpi(int32 dpi);
	void SetPaperSize(int32 width, int32 height);
	// bitmap size from ghostscript
	void SetBitmapSize(int32 width, int32 height);
	void Wait() { gs.Wait(); }
	void Stop() { gs.Stop(); }
};
#endif
