/*  
**   Copyright (C) 2001 Michael Pfeiffer
*/

#include "GlobalSettings.h"
#include <Message.h>
#include <FindDirectory.h>
#include <Path.h>
#include <File.h>

GlobalSettings::GlobalSettings() : 
	changed(false)
{
	SETTINGS(DEFINE_VARIABLE)
	STRING_SETTINGS(DEFINE_STRING_VARIABLE)
}

SETTINGS(DEFINE_SETTER)
SETTINGS(DEFINE_GETTER)

STRING_SETTINGS(DEFINE_STRING_SETTER)
STRING_SETTINGS(DEFINE_STRING_GETTER)

// BArchivable: 
GlobalSettings::GlobalSettings(BMessage *archive) {
	changed = false;

    SETTINGS(LOAD_SETTINGS)
    STRING_SETTINGS(LOAD_STRING_SETTINGS)
}

status_t GlobalSettings::Archive(BMessage *archive, bool deep) const {
	archive->AddString("class", "GlobalSettings");

	SETTINGS(STORE_SETTINGS)

	STRING_SETTINGS(STORE_STRING_SETTINGS)

	return B_OK;
}

BArchivable *GlobalSettings::Instantiate(BMessage *archive) {
	if (validate_instantiation(archive, "GlobalSettings")) {
		return new GlobalSettings(archive);
	} else
		return NULL;
}

// load settings from /boot/home/config/settings/<settingsFilename> if it exists
void GlobalSettings::Load(const char* filename) {
	BPath path(filename);
	if (path.InitCheck() == B_OK) {
		BFile file(path.Path(), B_READ_ONLY);
		if (file.InitCheck() == B_OK) {
			BMessage archive;
			archive.Unflatten(&file);
			GlobalSettings *s = (GlobalSettings*)GlobalSettings::Instantiate(&archive);

			if (s != NULL) {
				*this = *s; delete s;
			}
		}
	}
}

// save settings to settings file
void GlobalSettings::Save(const char* filename, bool force) {
BPath path(filename);
	if (HasChanged() || force) {
		BFile file(path.Path(), B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE);
		if (file.InitCheck() == B_OK) {
			BMessage archive;
			Archive(&archive);
			archive.Flatten(&file);
		}
	}
}

