/*  
    Copyright (C) 2001 Michael Pfeiffer
*/

#ifndef GLOBAL_SETTINGS_H
#define GLOBAL_SETTINGS_H

#include <String.h>
#include <Archivable.h>
#include <Point.h>
#include <Node.h>
#include <Bitmap.h>

#define DECLARE_VARIABLE(type, Type, method, name, value) type _##name;
#define DEFINE_VARIABLE(type, Type, method, name, value)  _##name = value;
#define DECLARE_SETTER(type, Type, method, name, value)   void Set##method(type name);
#define DECLARE_GETTER(type, Type, method, name, value)   type Get##method() const;
#define DEFINE_SETTER(type, Type, method, name, value)    void GlobalSettings::Set##method(type name) { changed = changed || (_##name == name); _##name = name; }
#define DEFINE_GETTER(type, Type, method, name, value)    type GlobalSettings::Get##method() const { return _##name; }
#define STORE_SETTINGS(type, Type, method, name, value)   archive->Add##Type(#name, _##name);
#define LOAD_SETTINGS(type, Type, method, name, value)  \
  if (B_OK != archive->Find##Type(#name, &_##name))     \
    _##name = value;


#define SETTINGS(do)                                                                                \
/*   Type   Be Type   Function Name Part      Member Variable Name                  Default Value */\
  do(int32,   Int32,  Dpi,                    dpi,                                              72) \
  do(int16,   Int16,  PaperWidth,             paperW,                                          595) \
  do(int16,   Int16,  PaperHeight,            paperH,                                          842) \
                                                                                                    \
  do(float,   Float,  WindowWidth,            windowWidth,                                     400) \
  do(float,   Float,  WindowHeight,           windowHeight,                                    600) \
  do(BPoint,  Point,  WindowPosition,         windowPos,                          BPoint(100, 100)) \
                                                                                                    \
  do(BPoint,  Point,  TraceWindowPosition,    traceWinPos,                        BPoint(100, 100)) \
  do(float,   Float,  TraceWinWidth,          traceWinWidth,                                   440) \
  do(float,   Float,  TraceWinHeight,         traceWinHeight,                                  404) \
                                                                                                    \
  do(bool,    Bool,   TraceAutoOpen,          traceAutoOpen,                                  true) \
  do(bool,    Bool,   TraceShowStdout,        traceStdout,                                    true) \
  do(bool,    Bool,   TraceShowStderr,        traceStderr,                                    true) \
  do(bool,    Bool,   TraceFloating,          traceFloating,                                  true) 

// This will generate setter and accessors in class GlobalSettings and their implementation.
// int GetChip() const;
// void SetChip(int );
  
#define WINDOW_SETTINGS(do)          \
  do(Window,         Window)         \
  do(TraceWindow,    TraceWin)

// Generates a setter and getter (but also requires WindowWidth and WindowHeight specified in the
// SETTINGS macro!!!).
// void GetWindowSize(float& width, float& height) const;
// void SetWindowSize(float width, float height);

#define DECLARE_STRING_VARIABLE(method, name, value) BString _##name;
#define DEFINE_STRING_VARIABLE(method, name, value)  _##name = value;
#define DECLARE_STRING_SETTER(method, name, value)   void Set##method(const char *string);
#define DECLARE_STRING_GETTER(method, name, value)   const char* Get##method() const;
#define DEFINE_STRING_SETTER(method, name, value)    void GlobalSettings::Set##method(const char *string) { changed = changed || (_##name == string); _##name = string; }
#define DEFINE_STRING_GETTER(method, name, value)    const char * GlobalSettings::Get##method() const { return _##name.String(); }
#define STORE_STRING_SETTINGS(method, name, value)   archive->AddString(#name, _##name.String());
#define LOAD_STRING_SETTINGS(method, name, value)  \
  if (B_OK != archive->FindString(#name, &_##name))     \
    _##name = value;

#define STRING_SETTINGS(do)                     \
  do(Device, device, "beos")    

#define SIZE_ACCESSOR(method, accessor) \
  void Get##method##Size(float &w, float &h) { w = Get##accessor##Width(); h = Get##accessor##Height(); } \
  void Set##method##Size(float w, float h)   { Set##accessor##Width(w);    Set##accessor##Height(h); }  
	
class _EXPORT GlobalSettings : public BArchivable {
	bool changed;
	
	SETTINGS(DECLARE_VARIABLE)
	STRING_SETTINGS(DECLARE_STRING_VARIABLE)
	
public:
	bool HasChanged() const { return changed; }

	// Initializes member variables with default values
	GlobalSettings(); 
	
	SETTINGS(DECLARE_SETTER)
	SETTINGS(DECLARE_GETTER)

	STRING_SETTINGS(DECLARE_STRING_SETTER)
	STRING_SETTINGS(DECLARE_STRING_GETTER)

	WINDOW_SETTINGS(SIZE_ACCESSOR)

	// BArchivable: 
	GlobalSettings(BMessage *archive);
	status_t Archive(BMessage *archive, bool deep = true) const;
	static BArchivable *Instantiate(BMessage *archive);

	// load settings from file
	void Load(const char* filename);
	// save settings to file (if force is true, write settings even there are no changes)
	void Save(const char* filename, bool force = false);
};

#endif
