// Written 2000 by Michael Pfeiffer
#ifndef GHOSTSCRIPT_WRAPPER_H
#define GHOSTSCRIPT_WRAPPER_H

#include <be/kernel/OS.h>
#include <be/support/String.h>
#include <be/interface/View.h>
#include <be/app/Looper.h>

// tries to load the Ghostscript Add-on
/*
extern int (* gsdll_revision) (char  **, char **, long *, long *);
extern int ( * gsdll_init) (GSDLL_CALLBACK, HWND, int argc, char ** argv);
extern int ( * gsdll_execute_begin) (void);
extern int ( * gsdll_execute_cont) (const char * str, int len);
extern int ( * gsdll_execute_end) (void);
extern int ( * gsdll_exit) (void);
extern int ( * gsdll_lock_device) (unsigned char *, int);
extern void (* gsdll_draw) (unsigned char *device, BView *view, BRect dest, BRect src);
*/

class Ghostscript {
	static bool gGhostscriptLoaded;

	sem_id fCmdSendSem;
	sem_id fCmdReceiveSem;
	static sem_id fWaitForPageSem;	
	BString fCommand;
	bool fIsRunning;

	static unsigned char *fBeDevice;
	static int gsdll_callback(int message, char *str, unsigned long count);
	static bool fStop;
	
	thread_id fThread;
	friend int32 ghostscript_thread(void *data);
	void Run();

	int32 fPaperWidth;
	int32 fPaperHeight;
	int32 fXDpi;
	int32 fYDpi;
	int32 fBitmapWidth;
	int32 fBitmapHeight;
	BString fDeviceName;

	static BLooper *looper;	
	void Init();
	void CalculateBitmapSize();
	
	static void SendMessage(uint32 what);
	static void SendMessage(BMessage *msg); 
public:
	static bool LoadGSDll();

	Ghostscript();
	~Ghostscript();	

	// Messages sent to looper
	enum {
		SYNC_NOTIFY = 'GWSy', // Update page
		PAGE_NOTIFY = 'GWPg', // End of page
		PAGE_CONT_NOTIFY = 'GWCo', // Sent after showpage
		SIZE_NOTIFY = 'GSSz', // Page Size (fields: int32 width, height)
		CMD_NOTIFY = 'GWCd',  // Command executed successfully
		CMD_ERROR_NOTIFY = 'GWEr', // Command returned with error
		EXIT_NOTIFY = 'GWEx'
	};

	// Parameters are used in the thread created by Start() and
	// must not change after Start() until Stop() is called
	void SetLooper(BLooper *looper) { this->looper = looper; };
	void SetPaperSize(int32 width, int32 height);
//	void GetPaperSize(int32 &w, int32 &h) { w = fPaperWidth; h = fPaperHeight; };
	void SetDpi(int32 xdpi, int32 ydpi);
	void GetDpi(int32 &x, int32 &y) { x = fXDpi; y = fYDpi; };
	void SetDevice(const char *dev); // e.g. "beos", "bealpha2" or "bealpha4"
	const char *GetDevice() { return fDeviceName.String(); };
	// GetBitmapSize() valid between a call of Start() and Stop()
//	void GetBitmapSize(int32 &width, int32 &height) { height = fBitmapHeight; width = bitmapWidth; };
	// Start Ghostscript interpreter
	void Start();
	// Stop Ghostscript interpreter 
	// (the latest sent command must have finished!!!)
	void Stop();
	
	// Is the Ghostscript thread running 
	bool IsRunning() { return fIsRunning; };
	// Send a command to Ghostscript
	bool Send(const char *cmd, int32 len);
	// Is the current command processing?
	bool IsProcessing();	
	// Wait to end of command (This will deadlock if gsdll_callback
	// waits to finish a page!!!)
	void Wait();
	// Is the interpreter waiting at the end of a page to begin a new page?
	// (When the PostScript command "showpage" is processed)
	bool WaitsForPage(); 
	// Let the interpreter finish a page and start a new if stop == false
	// otherwise halt the interpreter
	void NextPage(bool stop = false); 
	// Draw the Ghostscript bitmap into the view
	bool Draw(BView *view, BRect from, BRect to);
};

#endif